/**
 * $Id: FileResource.java,v 1.1 2001/07/27 02:07:00 pcj Exp $
 *
 * Copyright (C) 2001 Paul Cody Johnston - pcj@inxar.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 **/
package org.inxar.hotswap;

import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * Concrete implementation of <code>ProxyResource</code> that wraps a
 * <code>File</code>.
 *
 * @since 0.8.2
**/
class FileResource implements ProxyResource
{
    protected FileResource()
    {
    }

    FileResource(File file)
    {
	this.file = file;
    }

    public String toString()
    {
	return file.getAbsolutePath();
    }

    public long lastModified() throws IOException
    {
	return file.lastModified();
    }
    
    public long length() throws IOException
    {
	return file.length();
    }
    
    public boolean exists() throws IOException
    {
	return file.exists();
    }
    
    public InputStream getInputStream() throws IOException
    {
	return new FileInputStream(file);
    }

    protected File file;
}
