/**
 * $Id: JDK12ProxyClass.java,v 1.1 2001/07/27 02:07:00 pcj Exp $
 *
 * Copyright (C) 2001 Paul Cody Johnston - pcj@inxar.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 **/
package org.inxar.hotswap;

import java.io.File;
import java.io.IOException;
import java.util.Stack;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.ArrayList;

/**
 * Concrete <code>ProxyClass</code> implementation that does not
 * support dynamic proxy classes.
 *
 * @since 0.8.2
**/
class JDK12ProxyClass extends ProxyClass
{
    // ================================
    // Constructors 
    // ================================

    JDK12ProxyClass(ProxyClassLoader loader, String className)
    {
	super(loader, className);
	if (debug())
	    initDebug();
    }

    protected void initDebug()
    {
	log.debug("<init> New JDK1.2 ProxyClass created for "+className);
    }

    // ================================
    // ProxyClass Methods 
    // ================================

    public boolean isJDK13()
    {
	return false;
    }

    public Proxy newInstance()
    {
	return new KProxy(this);
    }

    public Proxy newInstance(Object[] args)
    {
	return new KProxy(this, null, args);
    }

    public Proxy newInstance(Class[] params, Object[] args)
    {
	return new KProxy(this, params, args);
    }

    public Proxy newInstanceH(Object h)
    {
	if (fatal())
	    log.fatal("Whoops!  The newInstanceH methods are reserved for JDK1.3 ProxyClasses only.");
	throw new UnsupportedOperationException("This is not a JDK1.3 ProxyClass.");
    }

    public Proxy newInstanceH(Object h, Object[] args)
    {
	if (fatal())
	    log.fatal("Whoops!  The newInstanceH methods are reserved for JDK1.3 ProxyClasses only.");
	throw new UnsupportedOperationException("This is not a JDK1.3 ProxyClass.");
    }

    public Proxy newInstanceH(Object h, Class[] params, Object[] args)
    {
	if (fatal())
	    log.fatal("Whoops!  The newInstanceH methods are reserved for JDK1.3 ProxyClasses only.");
	throw new UnsupportedOperationException("This is not a JDK1.3 ProxyClass.");
    }

    public Proxy newInstance(Object h)
    {
	if (fatal())
	    log.fatal("Whoops!  The newInstanceH methods are reserved for JDK1.3 ProxyClasses only.");
	throw new UnsupportedOperationException("This is not a JDK1.3 ProxyClass.");
    }

}



