/**
 * $Id: JavacCompiler.java,v 1.1 2001/07/27 02:07:00 pcj Exp $
 *
 * Copyright (C) 2001 Paul Cody Johnston - pcj@inxar.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 **/
package org.inxar.hotswap;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.OutputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import java.lang.reflect.Method;
import java.lang.reflect.Constructor;

import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * Concrete <code>ProxyCompiler</code> implementation that interfaces
 * with <code>sun.tools.javac.Main</code> (requires
 * <code>tools.jar</code> in the classpath).
 *
 * @since 0.8.2, renamed in 0.8.6
**/
public class JavacCompiler extends ProxyCompiler
{
    protected static final String COMMAND_NAME = "javac";
 
    // ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    // Implementation Note: This is all done using reflection in order
    // to not have to worry if user has tools.jar in the classpath.
    //
    // ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

    // ================================
    // Constructors 
    // ================================

    /**
     * Constructs a new <code>ProxyCompiler</code>.
     **/
    public JavacCompiler()
    {
	super();
    }

    public boolean initialize(Properties p)
    {
	if (!super.initialize(p))
	    return false;

	if (ctor == null) {
	    if (false)
		System.out.println
		    ("Cannot instantiate: cannot find sun.tools.javac.Main.  "+
		     "Please check your classpath for $JAVA_HOME/lib/tools.jar");
	    return false;
	} else 
	    return true;
    }
    
    // ================================
    // TEHC Methods
    // ================================

    // ================================
    // ProxyCompiler Methods 
    // ================================

    public synchronized ProxyCompileEvent compile(ProxyClass cls) 
    {
	Entry e = lookup(cls);
	String className = cls.getName();
	ProxyResource sourceFile = e.getSourcefile();
	ProxyResource classFile = e.getClassfile();

	// In this implementation we trust the compiler to tell us if
	// the compilation succeeded rather than doing file
	// modification tracking.

	boolean wasSuccessful = false;
	String out = "";
	String err = "";

	// Prepare the command line.
	String cmd = getCommand()
	    .append(' ')
	    .append(sourceFile.toString())
	    .toString();
	String[] argv = split(cmd, " ");
	
	ByteArrayOutputStream baos = new ByteArrayOutputStream();

	int rc = COMPILE_UNKNOWN;

	try {

	    // Create a new Javac compiler instance.
	    OutputStream os = new BufferedOutputStream(baos);
	    Object[] args = new Object[] { os, COMMAND_NAME };
	    Object javac = ctor.newInstance(args);

	    //wasSuccessful = new sun.tools.javac.Main(os, COMMAND_NAME).compile(argv);
	    wasSuccessful = ((Boolean)compile.invoke(javac, new Object[]{argv})).booleanValue();
	    rc = wasSuccessful ? COMPILE_SUCCESS : COMPILE_FAILURE;
	    
	} catch (Exception ioex) {
	    ioex.printStackTrace();
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    ioex.printStackTrace(pw);
	    err += sw.toString();
	    wasSuccessful = false;
	}

	if (!wasSuccessful)
	    out = "Compilation did not complete successfully.";
	
	err = baos.toString();

	ProxyCompileEvent evt = new ProxyCompileEvent
	    (this, className, "javac " + cmd.toString(), out, err, rc);

	if (debug())
	    log.debug(evt.toString());

	if (hasListeners())
	    fire(evt);

	return evt;
    }

    // ================================
    // Protected Methods 
    // ================================

    protected StringBuffer getCommand() 
    {
	StringBuffer cmd = new StringBuffer();
	
	if (opts != null) {
	    int count = 0;
	    Iterator i = opts.iterator();
	    while (i.hasNext()) {
		cmd.append(' ').append(i.next());
	    }
	}

	if (dst != null)
	    cmd.append(" -d ").append(dst);
	
	if (sourcepath != null)
	    cmd.append(" -sourcepath ").append(sourcepath);
	
	if (cps != null) {
	    cmd.append(" -classpath ");
	    int count = 0;
	    Iterator i = cps.iterator();
	    while (i.hasNext()) {
		if (count++ > 0)
		    cmd.append(PATHSEP);
		cmd.append(i.next());
	    }
	}

	return cmd;
    }

    protected String[] split(String s, String delim)
    {
	StringTokenizer st = new StringTokenizer(s, delim);
	String[] array = new String[st.countTokens()];
	int i=0;

	while (st.hasMoreTokens())
	    array[i++] = st.nextToken();

	return array;
    }

    // ================================
    // Static Fields 
    // ================================

    protected static Method compile;
    protected static Constructor ctor;

    static {
 	try {

	    // Fetch the compiler class
	    Class main = Class.forName("sun.tools.javac.Main");

	    // Setup the compile method
	    Class[] params = new Class[]{ String[].class }; 
	    compile = main.getMethod("compile", params);
	    
	    // Setup the constructor
	    params = new Class[]{ OutputStream.class, String.class };
	    ctor = main.getConstructor(params);

	} catch (Exception ex) {
	    compile = null;
	    ctor = null;
	} 
    }
}


