/**
 * $Id: ProxyClassClassLoader.java,v 1.1 2001/07/27 02:07:00 pcj Exp $
 *
 * Copyright (C) 2001 Paul Cody Johnston - pcj@inxar.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 **/
package org.inxar.hotswap;

import java.io.InputStream;
import java.io.IOException;

import java.util.Date;
import java.util.Map;
import java.util.TreeMap;

/**
 * Concrete <code>ClassLoader</code> implementation.
 *
 * @since 0.8.6
**/
class ProxyClassClassLoader extends ClassLoader
{
    protected static boolean DEBUG = true;

    // ================================
    // Constructors 
    // ================================

    ProxyClassClassLoader(ProxyClass proxyClass)
    {
	super(proxyClass.getParentClassLoader());
	this.proxyClass = proxyClass;
	this.classes = new TreeMap();
    }

    // ================================
    // TEHC Methods 
    // ================================

    /* remove underscore to enable */
    protected void _finalize() throws Throwable
    {
	Class cls = this.getClass();
	Class loader = cls.getClassLoader().getClass();

	System.out.println
	    ("Finalizing Object "+System.identityHashCode(this)+
	     " of Class "+cls.getName()+" "+System.identityHashCode(cls)+
	     " of ClassLoader "+loader.getName()+" "+System.identityHashCode(loader)+
	     ")");
    }

    // ================================
    // ClassLoader Methods
    // ================================
    
    protected synchronized Class findClass(String className) throws ClassNotFoundException
    {
	Class cls = (Class)classes.get(className);
	if (cls != null)
	    return cls;

	InputStream in = null;

	// This check minimizes the number of times the classfile
	// needs to be constructed since the proxyClass caches its
	// own.
	ProxyResource classFile = className.equals(proxyClass.getName())
	    ? proxyClass.getClassfile()
	    : proxyClass.getProxyCompiler().getClassfile(className);

	try {

	    long len = classFile.length();
	    byte[] buf = new byte[(int)len];
	    
	    in = classFile.getInputStream();
	    in.read(buf);
	    in.close(); 
	    in = null;
	    
	    cls = defineClass(className, buf, 0, buf.length);
	    resolveClass(cls);
	    
	    if (debug())
		log.debug("<findClass> class "+className+
			  " was loaded from "+classFile.toString()+
			  " dated "+new Date(classFile.lastModified()).toString());

	    /* okay, we have a new class. Store it and the return it
	       to the caller. */
	    classes.put(className, cls);

	    return cls;

	} catch (Exception ex) {
	    
	    ex.printStackTrace();
	    throw new ClassNotFoundException
		("Unable to load class "+className+": "+ex.getMessage());

	} finally {
	    if (in != null)
		try { in.close(); in = null; } 
		catch (Exception ex) {}
	}
    }

    // ================================
    // Debugging Methods 
    // ================================

    protected boolean debug()
    {
	if (lp == 0)
	    return false;

	if (lp < 0) 
	    initChannel();
	
	return (lp & ConsoleLog.PRI_DEBUG) != 0;
    }

    protected void initChannel()
    {
	if (proxyClass.getProxyClassLoader().hasLog()) {
	    log = proxyClass.getProxyClassLoader().getLog().newChannel("kcl", this);
	    lp = ConsoleLog.encode(log);
	} else {
	    lp = 0;
	}
    }

    // ================================
    // Instance Fields 
    // ================================

    protected Map classes;	
    protected ProxyClass proxyClass;

    // The log channel for this class
    protected ProxyLog.Channel log;	

    // The log priorities
    protected int lp = -1; 
}
