/**
 * $Id: ProxyHotSwapEvent.java,v 1.3 2001/06/25 20:30:23 pcj Exp $
 *
 * Copyright (C) 2001 Paul Cody Johnston - pcj@inxar.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 **/
package org.inxar.hotswap;

/**
 * <code>ProxyEvent</code> subtype fired by the
 * <code>ProxyClass</code> when a hotswap transaction has been
 * initiated.
 *
 * <P><code>ProxyHotSwapEvent</code> is a recent addition to the event
 * heirarchy.  It was added to the library in order for outside agents
 * to receive notification that a hotswap is about to be executed.
 * This is particularly important when there are change dependencies
 * outside of the sourcefile:classfile relationship.  <P>For example
 * if you wanted to implement a JSP compiler, you will most likely be
 * generating sourcefiles dynamically from a <code>file.jsp</code>
 * source. Therefore, the condition of whether to hotswap is dependent
 * upon the timestamp of <code>file.jsp</code>, not its
 * <code>file.java</code> counterpart.  Getting a
 * <code>ProxyHotSwapEvent</code> allows the user to check if the
 * <code>file.jsp</code> has been changed and regenerate a sourcefile
 * if necessary.
 *
 * @since 0.8.4
**/
public class ProxyHotSwapEvent extends ProxyEvent
{
    /**
     * Flexible subclass constructor.
    **/
    protected ProxyHotSwapEvent(Object source)
    {
	super(source);
    }

    /**
     * Standard constructor originates from the given
     * <code>ProxyClass</code> source.
    **/
    public ProxyHotSwapEvent(ProxyClass source)
    {
	super(source);
    }

    /**
     * Returns the associated <code>ProxyClass</code> to which the
     * source and classfiles are associated.  This method is
     * equivalent to <code>(ProxyClass)event.getSource()</code>.
    **/
    public ProxyClass getProxyClass()
    {
	return (ProxyClass)getSource();
    }

    /**
     * Prints a summary of the object swap.
    **/
    public String toString()
    {
	StringBuffer b = new StringBuffer();

	b.append("(ProxyHotSwapEvent: ")
	    .append(getProxyClass().getClassInstance().getName())
	    .append(')');

	return b.toString();
    }
}

