package org.inxar.hotswap;

import java.util.Properties;

/**
 * The <code>ProxyLog</code> interface is for applications that want
 * to hook their own logging implementation into HotSwap.  If logging
 * is enabled, HotSwap will pass log messages through this interface.
 *
 * @since 0.8.6
**/
public interface ProxyLog
{
    /**
     * Called directly after the <code>ProxyLog</code> has been
     * instantiated.
    **/
    boolean initialize(Properties p);

    /**
     * Creates a new logging channel for the given <code>Object</code>
     * under the given "common" label.
    **/
    Channel newChannel(String label, Object obj);

    /**
     * The <code>Channel</code> interface is the thing through which
     * actual log messages should be sent.
    **/
    interface Channel
    {
	/**
	 * Returns <code>true</code> if this <code>Channel</code>
	 * should report debugging messages.
  	**/
  	boolean doDebug();

	/**
	 * Returns <code>true</code> if this <code>Channel</code>
	 * should report info messages.
  	**/
  	boolean doInfo();

	/**
	 * Returns <code>true</code> if this <code>Channel</code>
	 * should report warning messages.
  	**/
  	boolean doWarn();

	/**
	 * Returns <code>true</code> if this <code>Channel</code>
	 * should report fatal error messages.
  	**/
	boolean doFatal();

	/**
	 * Sends a debugging message.
	**/
	void debug(Object msg);

	/**
	 * Sends an info message.
	**/
	void info(Object msg);

	/**
	 * Sends a warning message.
	**/
	void warn(Object msg);

	/**
	 * Sends a warning message detailed by the given
	 * <code>Throwable</code>.
	**/
	void warn(Object msg, Throwable t);

	/**
	 * Sends a fatal error message.
	**/
	void fatal(Object msg);

	/**
	 * Sends an error message detailed by the given
	 * <code>Throwable</code>.
	**/
	void fatal(Object msg, Throwable t);
    }
}



