/**
 * $Id: ProxyProperties.java,v 1.1 2001/07/27 02:07:00 pcj Exp $
 *
 * Copyright (C) 2001 Paul Cody Johnston - pcj@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package org.inxar.hotswap;

import java.util.Properties;
import java.util.List;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Collections;

/**
 * Subclass of <code>Properties</code> that transforms multiple-entry
 * keys into lists rather than clobbering old values.  Also transforms
 * new key entries into lowercase.
 *
 * @since 0.8.6
**/
class ProxyProperties extends Properties
{
    public Object put(Object key, Object val)
    {
	if (key instanceof String)
	    key = ((String)key).toLowerCase();

	Object o = get(key);
	if (o != null) {
	    if (o instanceof List) {
		List list = (List)o;
		list.add(val);
		return val;
	    } else {
		List list = new LinkedList();
		list.add(o);
		list.add(val);
		super.put(key, list);
		return val;
	    }
	} else {
	    return super.put(key, val);
	}
    }

    public void push(Object key, Object val)
    {
	if (key instanceof String)
	    key = ((String)key).toLowerCase();

	Object o = get(key);
	if (o != null) {
	    if (o instanceof List) {
		List list = (List)o;
		list.add(0, val);
	    } else {
		List list = new LinkedList();
		list.add(val);
		list.add(o);
		super.put(key, list);
	    }
	} else {
	    super.put(key, val);
	}
    }

    public static String getFirstProperty(String key, Properties p)
    {
	Object o = p.get(key);
	if (o instanceof List) {
	    List list = (List)o;
	    if (list.size() > 0)
		return (String)list.get(0);
	    else
		return null;
	} else if (o instanceof String)
	    return (String)o;
	else
	    return null;
    }

    public static List getProperties(String key, Properties p)
    {
	Object o = p.get(key);

	if (o == null)
	    return null;
	else if (o instanceof List)
	    return (List)o;
	else {
	    /* Whoops, JDK1.3 only */
	    //return new ListsingletonList(o);
	    List l = new ArrayList(1);
	    l.add(o);
	    return l;
	}
	
    }
}
