/**
 * $Id: ProxyResource.java,v 1.1 2001/07/27 00:32:16 pcj Exp $
 *
 * Copyright (C) 2001 Paul Cody Johnston - pcj@inxar.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 **/
package org.inxar.hotswap;

import java.io.InputStream;
import java.io.IOException;

/**
 * Abstraction of a <code>File</code> or some other entity that is
 * used to hold bytes of data.
 *
 * @since 0.8.6
 **/
public interface ProxyResource
{
    /**
     * Returns the time of last modification.
     **/
    long lastModified() throws IOException;
    
    /**
     * Returns the number of bytes in the <code>InputStream</code>.
     **/
    long length() throws IOException;
    
    /**
     * Returns <code>true</code> if the resource exists.
     **/
    boolean exists() throws IOException;

    /**
     * Returns an <code>InputStream</code> used to read the resource
     * data.
     **/
    InputStream getInputStream() throws IOException;

    /**
     * Returns a <code>String</code> representation of the
     * <code>Resource</code>.
     **/
    String toString();
}


