/**
 * $Id: ZipResource.java,v 1.1 2001/07/27 02:07:00 pcj Exp $
 *
 * Copyright (C) 2001 Paul Cody Johnston - pcj@inxar.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 **/
package org.inxar.hotswap;

import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * Concrete implementation of <code>ProxyResource</code> that wraps an
 * entry in a zip (or jar) file.
 *
 * @since 0.8.4
**/
class ZipResource extends FileResource
{
    ZipResource(ZipFile zipFile, ZipEntry zipEntry)
    {
	this.file = new File(zipFile.getName());
	this.zipFile = zipFile;
	this.zipEntry = zipEntry;
    }

    public String toString()
    {
	return "jar:file:"+file.getAbsolutePath()+"!/"+zipEntry.getName();
    }

    public long length() throws IOException
    {
	return zipEntry.getSize();
    }
    
    public InputStream getInputStream() throws IOException
    {
	ZipInputStream in = null;
	try {
	    in = 
		new ZipInputStream
		    (new BufferedInputStream
			(new FileInputStream(zipFile.getName())));
	    
	    ZipEntry e;
	    while (((e = in.getNextEntry())) != null) 
		if (e.getName().equals(zipEntry.getName()))
		    break;
	    
	    if (e == null)
		throw new ProxyRuntimeException("Cannot find ZipEntry "+toString());

	    int len = (int)length();
	    int off = 0;
	    byte[] buf = new byte[len];

	    int n;
	    while (len > 0) {
		n = in.read(buf, off, len);
		if (n == -1)
		    break;
		off += n;
		len -= n;
	    }

	    in.close();
	    in = null;

	    return new ByteArrayInputStream(buf);

	} catch (IOException ioex) {
	    ioex.printStackTrace();
	    return null;
	} finally {
	    if (in != null) 
		in.close();
	}
    }

    public InputStream _getInputStream() throws IOException
    {
	/* 

	   Would someone please explain why the following does NOT
	   work?  This would seem to be the easiest way to get the
	   input stream of the entry.  Indeed, the stream of a
	   classfile in a jar using this method *looks* byte-for-byte
	   identical to a FileInputStream on the same non-jarred
	   classfile, but trying to classload from it dies with UTF8
	   errors on my machine:

       java.lang.ClassFormatError: HelloWorld (Illegal UTF8 string in constant pool)
	   at java.lang.ClassLoader.defineClass0(Native Method)
	   at java.lang.ClassLoader.defineClass(ClassLoader.java:474)
	   at java.lang.ClassLoader.defineClass(ClassLoader.java:424)
	   at org.inxar.hotswap.ProxyClassClassLoader.findClass(ProxyClassClassLoader.java:101)
	   at java.lang.ClassLoader.loadClass(ClassLoader.java:295)
	   at java.lang.ClassLoader.loadClass(ClassLoader.java:252)
	   at org.inxar.hotswap.ProxyClass.newClassInstance(ProxyClass.java:555)
	   at org.inxar.hotswap.ProxyClass.getClassInstance(ProxyClass.java:252)
	   at org.inxar.hotswap.KProxy.hotswap_newInstance(KProxy.java:314)
	   at org.inxar.hotswap.KProxy.hotswap(KProxy.java:146)
	   at example.RunExample$Hotswapper.run(RunExample.java:140)
	   at java.lang.Thread.run(Thread.java:475)                                    

	   Whatup wit dat?

	   Also Note: BCEL does not complain, it parses just fine:

	*/

//  	if (zipEntry.getName().endsWith(".class")) {
//  	    de.fub.bytecode.classfile.JavaClass jcls = 
//  		new de.fub.bytecode.classfile.ClassParser
//  		    (zipFile.getInputStream(zipEntry), "").parse();
	    
//  	    System.out.println("jcls: "+jcls);
//  	}

	return zipFile.getInputStream(zipEntry);
    }

    protected ZipFile zipFile;
    protected ZipEntry zipEntry;
}
